Electric.Replication.PublicationManager (electric v1.4.13)
View SourceManages a PostgreSQL publication for a given Electric stack, tracking shapes and ensuring that the publication configuration matches the required set of relations that need to be published for the shapes to function correctly.
Includes periodic checks of the publication to ensure that it remains valid, and expires any shapes that are no longer valid due to schema changes or permission issues.
Summary
Functions
See Electric.Replication.PublicationManager.RelationTracker.add_shape/3.
See Electric.Replication.PublicationManager.Supervisor.child_spec/1.
See Electric.Replication.PublicationManager.RelationTracker.name/1.
See Electric.Replication.PublicationManager.RelationTracker.remove_shape/2.
See Electric.Replication.PublicationManager.Supervisor.start_link/1.
See Electric.Replication.PublicationManager.RelationTracker.wait_for_restore/2.