Electric.Replication.ShapeLogCollector.Supervisor (electric v1.4.13)

Supervisor for the ShapeLogCollector components.

Setting max_restarts to 0 as the supervisor only acts as a coordinator for starting and normal shutdowns, to preserve the ShapeLogCollector's death side effects in its supervision tree as before.

Summary

Functions

Returns a specification to start this module under a supervisor.

Callback implementation for Supervisor.init/1.

Functions

child_spec(init_arg)

Returns a specification to start this module under a supervisor.

See Supervisor.

init(opts)

Callback implementation for Supervisor.init/1.

name(stack_id)

start_link(opts)