Electric.Shapes.DynamicConsumerSupervisor (electric v1.6.2)

Copy Markdown View Source

Responsible for managing shape consumer processes.

Uses a set of DynamicSupervisors supervised by a parent DynamicSupervisor to take advantage of the fact that DynamicSupervisor terminates its children in parallel rather than one at a time.

This improves shutdown time because all consumer processes are effectively terminated simultaneously.

Summary

Functions

child_spec(arg)

Returns a specification to start this module under a supervisor.

See Supervisor.

name(name)

start_link(opts)

start_materializer(stack_id, config)

start_shape_consumer(stack_id, config)

start_snapshotter(stack_id, config)