Electric.Shapes.Supervisor (electric v1.4.13)

Supervisor responsible for the entire shape subsystem.

It starts up and supervises the processes that manage shapes (create/remove), keep the Postgres publication up to date, consume incoming transactions and write them to shape logs. It also supervisers the consumer supervisor which starts a new consumer process for each shape.

Summary

Functions

Returns a specification to start this module under a supervisor.

Functions

canary_name(stack_id)

child_spec(init_arg)

Returns a specification to start this module under a supervisor.

See Supervisor.

name(stack_ref)

reset_storage(opts)

start_link(opts)