Responsible for managing shape consumer processes.
Uses a set of DynamicSupervisors supervised by a parent DynamicSupervisor
to take advantage of the fact that DynamicSupervisor terminates its
children in parallel rather than one at a time.
This improves shutdown time because all consumer processes are effectively terminated simultaneously.
Summary
Functions
Returns a specification to start this module under a supervisor.